**    ModelEco : MODELisation en programmation mathmatique pour l'analyse ECOnomique de l'agriculure    **

*-----------------------------------------------------------------------------*
$Ontext
Nom              modelEco_elevageLait_3.gms
Sequence         2 - Le modle d'exploitation agricole
Langage GAMS     variable binaire, gdNb, LOOP
Type             LP
Description
L'exploitation laitire de M. Quatresabots est situe dans l'Ouest de la France.
C'est une exploitation de 50 ha dont 30 ha de terres labourables (sur lesquels l'exploitant peut faire du bl (pour vendre) ou/et du mas ensilage (pour l'alimentation de ses vaches laitires),
et 20 ha de prairies permanentes qui ne peuvent tre retournes (rgles du verdissement de la PAC) et qui sont ddies au pturage, ainsi qu' la production de foin et  d'enrubann.
Les vaches sont au pturage durant 6 mois (dbut avril - fin septembre). Durant la priode hivernale, elles sont nourries  base d'ensilage de mas (produit sur la ferme) et complmentes par des concentrs et tourteaux de soja (achet).
Les gnisses sont nourries par le pturage, le foin et l'enrubann.
Pour les vaches laitires, l'leveur peut faire varier un peu le rapport entre pturages et complments. On dfinit donc deux rgimes : le rgime qu'on appellera "  l'herbe "
dans lequel la part de pturage par rapport au mas et au soja est plus importante ; et un rgime qu'on appelle " intensif " dans lequel on augmente la part du soja et du mas. Le premier rgime permet une production moyenne par vache laitire et par an de 7000 litres (L);
le deuxime une production un peu plus importante de 7140 L
Enonc : http://www.supagro.fr/wordpress/modeleco/auto-formation/sequence2/cours3-simuler-une-politique-publique/activite-19-prime-pour-le-lait-non-produit/

$offtext

*-----------------------------------------------------------------------------*


*----------                          INDICES                        ----------*


SET
A     animaux bovins presents          /VL,Gen,Veau/
C     cultures activits vegetales     /ble,mais,prairie/
CA(C) cultures annuelles               /ble, mais/
F     fourrages produits               /ensilm,foin,enrub,pature/
CO    aliments concentrs achets      /soja/
R     type de ration                   /I,H/
sc    scenario                         /sc1*sc4/

SCALARS

TERlab  terre labourable disponible      /30/
pp      prairies permanentes             /20/
REF     taux de reforme                  /0.3/
PROL    prolificite par VL               /0.9/
Pxlait  prix du lait en  par litre      /0.27/
TxVendu taux de veaux vendus             /0.6/
rachat  montant du rachat du litre de lait non produit
gdNb                                     /1000000/


PARAMETER
PRIXC(C)   prix des cultures de vente en  par tonne     /ble 150/
COUT(C)    charges par culture en  par ha               /ble 600,mais 620,prairie 300/
RDTG(C)    rendement  culture de vente en tonne par ha   /ble 6/
COUTCO(CO) cout des concentrs  en  par tonne           /soja 360/

parameter mbc(c);
mbc(c)=prixc(c)*rdtg(c)-cout(c);

TABLE
RDTF(C,F) rendement fourrager en tonne par ha
          ensilm    foin    enrub   pature
mais       11
prairie              4.5     2.2      5

TABLE
BESF(A,R,F) besoins des animaux en fourrages en t par animal
          ensilm    foin    enrub   pature
VL.I       2                          1
Gen.I                2       2.6      2.5

VL.H       1                          1.5
Gen.H                2       2.6      2.5

TABLE
BESCO(A,R,CO) besoins des animaux en concentre en t par animal
             soja
VL.I         0.57
VL.H         0.45
;


PARAMETER
PRIXA(A)   prix de vente en  par animal            /VL 800, Veau 170/
PL(A,R)    productivit en litre par VL             /VL.I 7140, VL.H 7000/
Charges(A) chagres de structure et oprationnelles  /VL 1300/

parameter coutalim,coutaliml;
coutalim(A,R)=sum(co,besco(A,R,co)*coutco(CO));

*----------                          MODELE                        ----------*

VARIABLES
Z
X(C)
N(A, R)
laitTot
;

positive variables
X,N, laitTot
;

binary variable
B
;

Equations
Objectif       fonction objectif
Terre          contrainte de surface pour les terres labourables
Pperm          contrainte de surface pour les prairies permanentes
Ration(F)      contrainte d'alimentation
Reforme        contrainte de dmographie (vaches rformes)
Demog          contrainte de dmographie
laitCalcul     calcul la quantit totale de lait produit
prime
;

Objectif ..      Z =e= sum(c,X(c)*mbc(C))+sum((A,R),N(A,R)*PL(A,R)*pxlait)
                  +sum(R,N('Veau',R)*prixa('Veau'))
                  +sum(R,N('VL',R)*REF*prixa('VL') )
                  -sum((A,R),N(A,R)*coutalim(A,R))
                  -sum((A,R),N(A,R)*Charges(A))
                 +(400000-laitTot)*rachat*B
;
Terre ..         sum(ca,X(ca)) =l= terlab;
Pperm..          x('prairie')=l=PP;
Ration(F) ..     sum((A,R),N(A,R)*BESF(A,R,F)) =l= sum(C,X(C)*RDTF(C,F));
Reforme(R) ..    N('Gen',R) =e= N('VL',R)*ref;
Demog(R)   ..    N('veau',R)+N('Gen',R) =e= N('VL',R)*prol;
laitCalcul..     sum((A,R),N(A,R)*PL(A,R)) =e= laitTot;
prime..          400000-laitTot =l= 0.2*400000 + (1-B)*gdNb;


model lait /all/;

*----------                        SOLUTION                         ----------*

parameter RESULT(*,*,*) tableau de rsultat ;

parameter rachatSc(sc)
/sc1   0.0
 sc2   0.02
 sc3   0.03
 sc4   0.04
/

LOOP(sc,
rachat=rachatSc(sc);

*** on fait une rinitialisation des variables avant chaque rsolution ***
LOOP(R,
LOOP(A,
N.l(A,R)=0
))
LOOP(C,
X.l(C)=0
) ;
Z.l=0;
laitTot.l=0;

solve lait using MINLP maximizing Z;

result("pxLait","",sc)=pxLait;
result(A,R,sc)=N.l(A,R);
result(C,"",sc)=X.l(C);
result("revenu","",sc)=Z.l;
result("prodLait","",sc)=laitTot.l;
result("rachat?","",sc)=B.l;
result("rachat","",sc)=rachat;
)

display RESULT;